import 'dart:developer';
import 'dart:isolate';

import 'package:flutter_foreground_task/flutter_foreground_task.dart';
import 'package:vivanta_connect_example/vivanta.dart';
import 'package:vivanta_connect_flutter/helpers/vivanta_sync.dart';
import 'package:vivanta_connect_flutter/services/preferences.dart';

class VivantaSyncTaskHandler extends TaskHandler {
  SendPort? _sendPort;
  VivantaSync? _vivantaSync;

  // Called when the task is started.
  @override
  void onStart(DateTime timestamp, SendPort? sendPort) async {
    _sendPort = sendPort;
    final userId = await Preferences().getExternalUserId();
    _vivantaSync = VivantaSync(
      apiKey: Vivanta.apiKey,
      customerId: Vivanta.customerId,
      externalUserId: userId,
      isFromForegroundService: true,
    );
  }

  // Called every [interval] milliseconds in [ForegroundTaskOptions].
  @override
  void onRepeatEvent(DateTime timestamp, SendPort? sendPort) async {
    // Send data to the main isolate.
    if (_vivantaSync == null) {
      final userId = await Preferences().getExternalUserId();
      _vivantaSync = VivantaSync(
        apiKey: Vivanta.apiKey,
        customerId: Vivanta.customerId,
        externalUserId: userId,
        isFromForegroundService: true,
      );
    }

    await _vivantaSync!.executeAll();
    log('Vivanta Connect Example >> Task is onRepeatEvent at ${DateTime.now().toIso8601String()}');
    sendPort?.send('OK');
  }

  // Called when the notification button on the Android platform is pressed.
  @override
  void onDestroy(DateTime timestamp, SendPort? sendPort) async {}

  // Called when the notification button on the Android platform is pressed.
  @override
  void onNotificationButtonPressed(String id) {
    log('onNotificationButtonPressed >> $id');
  }

  // Called when the notification itself on the Android platform is pressed.
  //
  // "android.permission.SYSTEM_ALERT_WINDOW" permission must be granted for
  // this function to be called.
  @override
  void onNotificationPressed() {
    // Note that the app will only route to "/resume-route" when it is exited so
    // it will usually be necessary to send a message through the send port to
    // signal it to restore state when the app is already started.
    FlutterForegroundTask.launchApp('/');
    _sendPort?.send('onNotificationPressed');
  }
}
